<?php
namespace Drupal\asc_courses;

// use Consolidation\OutputFormatters\StructuredData\RowsOfFields;
// use Drush\Commands\DrushCommands;
use \Drupal\Core\Database\Database;
use \Drupal\Core\Config;
use \Drupal\node\Entity\Node;

class AscCoursesImporter {
  public $debug = 0;
  public $config;
  public $api;

  /**
   * {@inheritdoc}
   */
  public function __construct($config = null) {
    // Get settings
    if (!empty($config)) {
      $this->config = $config;
    }
    else {
      $this->config = $config = \Drupal::service('config.factory')->get('asc_courses.settings');
    }
  }

  public function importCourseNodes($courses_data) {
    if($this->debug) echo "importCourseNodes()\n";
    $i = 0;
    foreach($courses_data->getCourseCatalogResponse->catalog->course as $course_data) {
      $node_storage = \Drupal::entityTypeManager()->getStorage('node');
      // if($this->debug) echo "$i catalog-nbr: " . $course_data->{'catalog-nbr'} . "\n";
      if($i < 13) {
        // if($this->debug > 1) echo "importCourseNodes() - " . print_r($course_data, true);

        // look up existing node
        $node_query = \Drupal::entityQuery('node')
          ->condition('field_eip_id', $course_data->{'crse-id'}, '=');
        $existing_node = $node_query->execute();

        if(empty($existing_node)) {
          $this->createCourseNode($node_storage, $course_data);
        }
        else {
          // update existing node
          $existing_nid = array_shift($existing_node);
          $this->updateCourseNode($node_storage, $existing_nid, $course_data);
          // if($this->debug) echo "\n";
        }
        // $i++;
      }
    }
  }

  public function createCourseNode($node_storage, $course_data) {
    if($this->debug) echo "createCourseNode() - create new node\n";
    // $course_node = Node::create([
    $course_node = $node_storage->create(array(
      'type'                        => 'course',
      'title'                       => $course_data->{'course-title-long'},
      'field_course_description'    => [$course_data->{'descrlong'}],
      'field_course_number'         => [$course_data->{'catalog-nbr'}],
      'field_credit_hours'          => [$course_data->{'acad-prog'}],
      'field_subject_abbreviation'  => [$course_data->{'subject'}],
      'field_eip_id'                => [$course_data->{'crse-id'}],
    ));
    $course_node->save();
    // $new_course_nid = $course_node->get('nid')->getValue()[0]['value'];
    $new_course_nid = $course_node->id();
    // if($this->debug) echo "createCourseNode() - $new_course_nid\n";
    if($this->debug) echo "createCourseNode() - New course nid[$new_course_nid] catalog[" . $course_data->{'catalog-nbr'} . "]\n";
  }

  public function updateCourseNode($node_storage, $nid, $course_data) {
    if($this->debug) echo "updateCourseNode() - Existing nid: $nid\n";

    $course_node = $node_storage->load($nid);

    // Compare values and update if necessary
    $content_changed = false;

    // $title = $course_node->get('title')->getValue()[0]['value'];
    // echo "title: " . print_r($title, true);
    $title = $course_node->title->value;
    if ($title != $course_data->{'course-title-long'}) {
      if($this->debug) {
        echo "### Title changed..\n";
        echo "Node title: $title\n";
        echo "API title: " . $course_data->{'course-title-long'} . "\n";  
      }
      $content_changed = true;
      // $course_node->setTitle($course_data->{'course-title-long'});
      $course_node->title->value = $course_data->{'course-title-long'};
    }

    $descrption = $course_node->field_description->value;
    // echo "descrption: $descrption\n";
    if ($descrption != $course_data->{'descrlong'}) {
      if($this->debug) {
        echo "### descrption changed..\n";
        echo "Node descrption: $descrption\n";
        echo "API descrption: " . $course_data->{'descrlong'} . "\n";  
      }
      $content_changed = true;
      $course_node->descrption->value = $course_data->{'descrlong'};
    }

    // $course_number = $course_node->get('field_course_number')->getValue()[0]['value'];
    // echo "course_number: " . print_r($course_number, true);
    $course_number = $course_node->field_course_number->value;
    if ($course_number != $course_data->{'catalog-nbr'}) {
      if($this->debug) {
        echo "### course_number changed..\n";
        echo "Node course_number: $course_number\n";
        echo "API course_number: " . $course_data->{'catalog-nbr'} . "\n";  
      }
      $content_changed = true;
      // $course_node->set('field_course_number', $course_data->{'catalog-nbr'});
      $course_node->field_course_number->value = $course_data->{'catalog-nbr'};
    }

    $credit_hours = $course_node->field_credit_hours->value;
    if ($credit_hours != $course_data->{'acad-prog'}) {
      if($this->debug) {
        echo "### credit_hours changed..\n";
        echo "Node credit_hours: $credit_hours\n";
        echo "API credit_hours: " . $course_data->{'acad-prog'} . "\n";  
      }
      $content_changed = true;
      $course_node->field_credit_hours->value = $course_data->{'acad-prog'};
    }

    $subj_abbrev = $course_node->field_subject_abbreviation->value;
    if ($subj_abbrev != $course_data->{'subject'}) {
      if($this->debug) {
        echo "### subj_abbrev changed..\n";
        echo "Node subj_abbrev: $subj_abbrev\n";
        echo "API subj_abbrev: " . $course_data->{'subject'} . "\n";  
      }
      $content_changed = true;
      $course_node->field_subject_abbreviation->value = $course_data->{'subject'};
    }

    if($content_changed) {
      if($this->debug) echo "\nContent has changed.. save updated node.\n\n";
      $course_node->save();
    }
  }


  public function fetchAndImportAll() {
    // $this->config = \Drupal::service('config.factory')->get('asc_courses.settings');
    if(!isset($this->api)) $this->api = new AscCoursesApi($this->config);

    $config_dorgs = $this->config->get('asc_courses.dept_org');

    $dorgs = explode(',', trim($config_dorgs));

    foreach($dorgs as $dorg) {
      $dorg = trim($dorg);
      if ($this->debug) echo "D-org: $dorg\n";
      $db_data = $this->loadSubjectDataFromDatabase($dorg);
      if(empty($db_data)) {
        $courses_data = $this->api->fetchSubjectCourses($dorg);
      }
      else {
        $courses_data = $db_data;
      }
      
      $this->importCourseNodes($courses_data);
    }
  }

  public function loadSubjectDataFromDatabase($dorg) {
    $connection = \Drupal::database();
    $course_info_query = $connection->select('asc_course_data', 'acd')
      // ->fields('acd', array('id', 'date', 'dept_org'))
      ->fields('acd')
      ->condition('dept_org', $dorg, "=")
      ->orderBy('date', 'DESC')
      ->range(0, 1);
    $course_info_result = $course_info_query->execute();
    // $result_count = $course_info_result->rowCount();
    // echo "Rows: $result_count\n";
    // $rows = $course_info_result->fetchAllAssoc('id', \PDO::FETCH_ASSOC);
    if($row = $course_info_result->fetchAssoc()) {
      // print_r($row);
      $data_date = $row['date'];
      $data_age = time() - $data_date;
      if($this->debug) echo "Data date: $data_date - " . date("Y-m-d h:i:s", $data_date) . " - $data_age seconds old\n";

      if($data_age < 3600) {
        if($this->debug) echo "Data is less than an hour old.. reusing data from database.\n";
        $courses_json = $row['raw_json'];
        // $len = strlen($courses_json);
        if($this->debug) echo "JSON Length:" . strlen($courses_json) . "\n";
        // echo substr($courses_json, 0, 1024) . "\n\n";
        $json_data = json_decode($courses_json);
        // print_r($json_data);
        return $json_data;
      }
      else {
        return false;
      }
    }
    else {
      return false;
    }
  }


}
